﻿Imports System.Collections.Specialized
Imports System.IO
Imports System.Globalization
Imports Microsoft.TeamFoundation.Client
Imports Microsoft.TeamFoundation.WorkItemTracking.Client

Class Application

    ' Application-level events, such as Startup, Exit, and DispatcherUnhandledException
    ' can be handled in this file.

    ' The name of the application folder.  This folder is used to save the files 
    ' for this application such as the photos, stories and family data.
    Private Const ApplicationFolderName As String = "TFS Sticky Buddy"

    'The main connection to team server for the whole application
    ' The FamilyCollection and Family fields are accessed from the same thread,
    ' so suppressing the CA2211 code analysis warning.
    <System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2211:NonConstantFieldsShouldNotBeVisible")> _
    Private Shared m_TFServer As TeamFoundationServer = Nothing

    Public Shared Property TFServer() As TeamFoundationServer
        Get
            Return m_TFServer
        End Get
        Set(ByVal value As TeamFoundationServer)
            m_TFServer = value
            m_TFServer.EnsureAuthenticated()
            m_WorkItemStore = DirectCast(value.GetService(GetType(WorkItemStore)), WorkItemStore)
            m_SelectedProject = m_WorkItemStore.Projects(0)
        End Set
    End Property

    <System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2211:NonConstantFieldsShouldNotBeVisible")> _
    Private Shared m_WorkItemStore As WorkItemStore = Nothing

    <System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2211:NonConstantFieldsShouldNotBeVisible")> _
    Public Shared ReadOnly Property WorkItemStore() As WorkItemStore
        Get
            Return m_WorkItemStore
        End Get
    End Property

    <System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2211:NonConstantFieldsShouldNotBeVisible")> _
    Private Shared m_SelectedProject As Project = Nothing

    <System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2211:NonConstantFieldsShouldNotBeVisible")> _
    Public Shared Property SelectedProject() As Project
        Get
            Return m_SelectedProject
        End Get
        Set(ByVal value As Project)
            m_SelectedProject = value
        End Set
    End Property

    <System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2211:NonConstantFieldsShouldNotBeVisible")> _
    Private Shared m_SelectedWITs As New System.Collections.Generic.Dictionary(Of String, WorkItemType)

    <System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2211:NonConstantFieldsShouldNotBeVisible")> _
    Public Shared ReadOnly Property SelectedWITs() As System.Collections.Generic.Dictionary(Of String, WorkItemType)
        Get
            Return m_SelectedWITs
        End Get
    End Property

    <System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2211:NonConstantFieldsShouldNotBeVisible")> _
    Private Shared m_DisplayBy As Node.TreeType = Node.TreeType.Area

    <System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2211:NonConstantFieldsShouldNotBeVisible")> _
Public Shared Property DisplayBy() As Node.TreeType
        Get
            Return m_DisplayBy
        End Get
        Set(ByVal value As Node.TreeType)
            m_DisplayBy = value
        End Set
    End Property


    <System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2211:NonConstantFieldsShouldNotBeVisible")> _
Public Shared ItemWrappers As New ItemWrapperCollection

    ''' <summary>
    ''' Occurs when the application starts.
    ''' </summary>
    <System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")> _
    Protected Overrides Sub OnStartup(ByVal e As StartupEventArgs)
        Dim appSettings As Settings = Settings.Default

        If Not String.IsNullOrEmpty(appSettings.Skin) Then
            Try
                Dim rd As ResourceDictionary = New ResourceDictionary()
                Dim uri As New Uri(appSettings.Skin, UriKind.Relative)
                Dim Component As ResourceDictionary = CType(Application.LoadComponent(uri), ResourceDictionary)
                rd.MergedDictionaries.Add(Component)
                Application.Current.Resources = rd
            Catch ex As Exception

            End Try
        End If

        MyBase.OnStartup(e)
    End Sub


    ''' <summary>
    ''' Occurs when the application exits.
    ''' </summary>
    Protected Overrides Sub OnExit(ByVal e As ExitEventArgs)
        ' Save the file automatically
        'If (Family.IsDirty & Not String.IsNullOrEmpty(FamilyCollection.FullyQualifiedFilename)) Then
        '    FamilyCollection.Save()
        'End If
        MyBase.OnExit(e)
    End Sub

    ''' <summary>
    ''' Gets the collection of skins
    ''' </summary>
    Public Shared ReadOnly Property Skins() As NameValueCollection
        Get
            Dim skinlist As NameValueCollection = New NameValueCollection()
            Dim SkinFolder As String = My.Resources.Skins
            For Each folder As String In Directory.GetDirectories(SkinFolder)
                For Each file As String In Directory.GetFiles(folder)
                    Dim fileInfo As FileInfo = New FileInfo(file)
                    If String.Compare(fileInfo.Extension, My.Resources.XamlExtension, True, CultureInfo.InvariantCulture) = 0 Then
                        ' Use the first part of the resource file name for the menu item name.
                        skinlist.Add(fileInfo.Name.Remove(fileInfo.Name.IndexOf(My.Resources.ResourcesString)), Path.Combine(folder, fileInfo.Name))
                    End If
                Next
            Next
            Return skinlist
        End Get
    End Property

    ''' <summary>
    ''' Return the animation duration. The duration is extended
    ''' if special keys are currently pressed (for demo purposes)  
    ''' otherwise the specified duration is returned. 
    ''' </summary>
    Public Shared Function GetAnimationDuration(ByVal milliseconds As Double) As TimeSpan
        If Keyboard.IsKeyDown(Key.F12) Then
            Return TimeSpan.FromMilliseconds(milliseconds * 5)
        Else
            Return TimeSpan.FromMilliseconds(milliseconds)
        End If
    End Function



End Class
